package farmacie_con_liste;

import java.util.*;

public class Farmacia {
	
	private static final int MAX_MEDICINALI = 2000;
	private static final int MIN_CONFEZIONI = 5;
	
	private String codice;
	private String indirizzo;
	private List<Medicinale> prontuario;
	private InfoMedicinale[] disponibilita;
	private int numMedicinali;
	
	public Farmacia() {
		this("","",new ArrayList<Medicinale>(),new InfoMedicinale[MAX_MEDICINALI],0);
	}
	
	public Farmacia(String codice, String indirizzo, List<Medicinale> prontuario, InfoMedicinale[] disponibilita, int n) {
		this.codice = codice;
		this.indirizzo = indirizzo;
		this.prontuario = prontuario;
		this.disponibilita = disponibilita;
		this.numMedicinali = n;
	}
	
	// punto 1)
	public int disponibilitaMedicinale(String codMedicinale) {
		int numConfezioni = -1;
		boolean trovato = false;
		for(int i=0; i<numMedicinali && !trovato; i++)
			if(disponibilita[i].getCodiceMedicinale().equals(codMedicinale)) {
				trovato = true;
				numConfezioni = disponibilita[i].getNumConfezioni();
			}
		return numConfezioni;
	}
	
	
	// punto 2) 
	public boolean medicinalePresente(String codMedicinale) {
		boolean presente = false;
		Iterator<Medicinale> i = prontuario.iterator();
		while(i.hasNext() && !presente)
			if(i.next().getCodice().equals(codMedicinale)) 
				presente = true;
		return presente;	
	}
	
	/*
	// punto 2) versione con metodi per l'accesso posizionale
	public boolean medicinalePresente(String codMedicinale) {
		boolean presente = false;
		for(int i=0; i<prontuario.size() && !presente; i++)
			if(prontuario.get(i).getCodice().equals(codMedicinale)) 
				presente = true;
		return presente;	
	}
	*/
		
	// punto 3)
	public double percentualeOmeopaticiPerFormato(String format) {
		int totOmeopatici= 0;
		for(int i=0; i<prontuario.size(); i++)
			if(prontuario.get(i) instanceof MedicinaleNonTradizionale)
				if(((MedicinaleNonTradizionale)prontuario.get(i)).getTipologia().toLowerCase().equals("omeopatico")
						&& prontuario.get(i).getFormato().equalsIgnoreCase(format))
					totOmeopatici++;
		return (double)totOmeopatici/prontuario.size()*100;
	}
	
	// punto 4)
	public boolean aggiungiMedicinale(Medicinale med, int numConf) {
		boolean nuovo = false;
		if(!medicinalePresente(med.getCodice())) {
			nuovo = true;
			prontuario.add(med);
			InfoMedicinale info = new InfoMedicinale(med.getCodice(),numConf);
			if(numMedicinali < MAX_MEDICINALI)
				disponibilita[numMedicinali++] = info;
			else
				System.out.println("Errore: spazio non sufficiente per memorizzare i dati sulla disponibilit");
		} else {
			int i = trovaInfoMedicinale(med.getCodice());
			if(i != -1) //  stato trovato il medicinale nel vettore delle disponibilit
				disponibilita[i].aggiungiConfezioni(numConf);
		}
		return nuovo;
	}
	
	// metodo privato di utilit per il metodo 4)
	private int trovaInfoMedicinale(String codiceMed) {
		int posizione = -1;
		boolean trovato = false;
		for(int i=0; i<numMedicinali && !trovato; i++)
			if(disponibilita[i].getCodiceMedicinale().equals(codiceMed)) {
				trovato = true;
				posizione = i;
			}
		return posizione;
	}
	
	// punto 5)
	public List<Medicinale> riordino(){
		List<Medicinale> daRiordinare = new ArrayList<Medicinale>();
		for(int i=0; i<numMedicinali; i++)
			if(disponibilita[i].getNumConfezioni()<MIN_CONFEZIONI)
				daRiordinare.add(trovaMedicinale(disponibilita[i].getCodiceMedicinale()));
		return daRiordinare;
	}
	
	// metodo privato di utilit per il metodo 5)
	private Medicinale trovaMedicinale(String codiceMed) {
		Medicinale medicinale = null;
		boolean trovato = false;
		for(int i=0; i<prontuario.size() && !trovato; i++)
			if(prontuario.get(i).getCodice().equals(codiceMed)) {
				trovato = true;
				medicinale = prontuario.get(i);
			}
		return medicinale;
	}

}
